/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: GetRulePackagesCommand.as 4876 2009-08-11 14:22:08Z nicoulaj $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.rules.manager.command
{
import com.adobe.cairngorm.commands.ICommand;
import com.adobe.cairngorm.control.CairngormEvent;
import com.adobe.cairngorm.control.CairngormEventDispatcher;

import flash.utils.getQualifiedClassName;

import mx.collections.ArrayCollection;
import mx.rpc.IResponder;
import mx.rpc.events.FaultEvent;
import mx.rpc.events.ResultEvent;

import org.ow2.jasmine.kerneos.common.event.KerneosNotificationEvent;
import org.ow2.jasmine.monitoring.eos.common.controls.ServerSideException;
import org.ow2.jasmine.monitoring.eos.common.events.ServerSideExceptionEvent;
import org.ow2.jasmine.monitoring.eos.rules.common.event.SynchronizedRulePackageEvent;
import org.ow2.jasmine.monitoring.eos.rules.common.model.domain.SynchronizedRulePackage;
import org.ow2.jasmine.monitoring.eos.rules.common.model.domain.SynchronizedRulePackageState;
import org.ow2.jasmine.monitoring.eos.rules.common.vo.rulepackage.IRulePackageVO;
import org.ow2.jasmine.monitoring.eos.rules.manager.business.*;
import org.ow2.jasmine.monitoring.eos.rules.manager.model.RuleManagerModelLocator;
import org.ow2.jasmine.monitoring.eos.rules.manager.model.domain.RuleManagerModel;

/**
* Get the rule packages
* 
* @author Julien Nicoulaud
*/
[Event(name="serverSideException", type="org.ow2.jasmine.monitoring.eos.common.events.ServerSideExceptionEvent")] 
public class GetRulePackagesCommand implements ICommand, IResponder
{

    // =========================================================================
    // Execution
    // =========================================================================
    
    /**
    * Send the event to the java side, using the business layer of the pattern
    */
    public function execute(event:CairngormEvent):void
    {
        // Retrieve the command delegate
        var delegate:IGetRulePackagesDelegate = 
        	RuleManagerModelLocator.getInstance().getGetRulePackagesDelegate();
        
        // Set this object as a responder
        delegate.responder = this;
        
        // Retrieve the component model
        var model:RuleManagerModel = 
        	RuleManagerModelLocator.getInstance().ruleManagerModel;
        
        // Execute a server-side method thanks to the delegate
        delegate.getRulePackages(model.ruleEngineName);
    }


    // =========================================================================
    // Result handling
    // =========================================================================

    /**
    * Get the result of the java side. this method is called on each event
    * returned from the Java side.
    */
    public function result(event : Object):void
    {
        // Retrieve the model
        var model:RuleManagerModel = 
        	RuleManagerModelLocator.getInstance().ruleManagerModel;
        	
        // Store the result in the model
        var rulePackages : ArrayCollection =
            ((event as ResultEvent).result as ArrayCollection);
        
        // Dispatch a notification to Kerneos
        CairngormEventDispatcher.getInstance().dispatchEvent(
            new KerneosNotificationEvent(KerneosNotificationEvent.KERNEOS_NOTIFICATION + model.componentID,
                                         (getQualifiedClassName(this).split("::"))[1] + ": Loading/reloading the rule packages",
                                         KerneosNotificationEvent.DEBUG)
        );
                
        // Clear the old packages list (tricky: only way to make sure a
        // CollectionEvent is dispatched for each entry)
        var numOldPackages:int = model.rulePackages.length;
        for(var i:int = 0; i<numOldPackages; i++) {
            model.rulePackages.removeItemAt(0);
        }
        
        // Re-fill the packages list with the new ones
        for each (var rulePackage : IRulePackageVO in rulePackages) {
            
            // Wrap the rule package in a SynchronizedRulePackage
            var newSynchronizedRulePackage : SynchronizedRulePackage =
                new SynchronizedRulePackage(rulePackage,SynchronizedRulePackageState.SYNCHRONIZED);
            
            // Add it to the collection
        	model.rulePackages.addItem(newSynchronizedRulePackage);
        
            // Launch a test compilation
            CairngormEventDispatcher.getInstance().dispatchEvent(
                new SynchronizedRulePackageEvent(SynchronizedRulePackageEvent.COMPILE_RULE_PACKAGE + model.componentID,
                                                 newSynchronizedRulePackage)
            );
        }
        
        // Mark the pending "Save" operations as cancelled
        model.pendingSaveOperations = 0;
    }


    // =========================================================================
    // Fault handling
    // =========================================================================
    
    /**
    * Handle fault messages
    */
    public function fault(event : Object) : void
    {
        // Retrieve the fault event
        var faultEvent : FaultEvent = FaultEvent(event);

        // Tell the view and let it handle this
        var serverSideExceptionEvent : ServerSideExceptionEvent =
            new ServerSideExceptionEvent(
                "serverSideException",
                new ServerSideException("Error while retrieving the rule packages",
                                        "The operation could not be performed."
                                        + "\n" + faultEvent.fault.faultCode
                                        + "\n" + faultEvent.fault.faultString,
                                        faultEvent.fault.getStackTrace()));
        CairngormEventDispatcher.getInstance().dispatchEvent(serverSideExceptionEvent);
    }

}
}
